diff --git a/tests/lib/connector/sabre/quotaplugin.php b/tests/lib/connector/sabre/quotaplugin.php
new file mode 100644
index 0000000000000000000000000000000000000000..1186de28742bdf76c502344b416ce44b915c5841
--- /dev/null
+++ b/tests/lib/connector/sabre/quotaplugin.php
@@ -0,0 +1,101 @@
+<?php
+/**
+ * Copyright (c) 2013 Thomas Müller <thomas.mueller@tmit.eu>
+ * This file is licensed under the Affero General Public License version 3 or
+ * later.
+ * See the COPYING-README file.
+ */
+
+class Test_OC_Connector_Sabre_QuotaPlugin extends PHPUnit_Framework_TestCase {
+
+	/**
+	 * @var Sabre_DAV_Server
+	 */
+	private $server;
+
+	/**
+	 * @var OC_Connector_Sabre_QuotaPlugin
+	 */
+	private $plugin;
+
+	public function setUp() {
+		$this->server = new Sabre_DAV_Server();
+		$this->plugin = new OC_Connector_Sabre_QuotaPlugin();
+		$this->plugin->initialize($this->server);
+	}
+
+	/**
+	 * @dataProvider lengthProvider
+	 */
+	public function testLength($expected, $headers)
+	{
+		$this->server->httpRequest = new Sabre_HTTP_Request($headers);
+		$length = $this->plugin->getLength();
+		$this->assertEquals($expected, $length);
+	}
+
+	/**
+	 * @dataProvider quotaOkayProvider
+	 */
+	public function testCheckQuota($quota, $headers)
+	{
+		$this->plugin->fileView = $this->buildFileViewMock($quota);
+
+		$this->server->httpRequest = new Sabre_HTTP_Request($headers);
+		$result = $this->plugin->checkQuota('');
+		$this->assertTrue($result);
+	}
+
+	/**
+	 * @expectedException Sabre_DAV_Exception_InsufficientStorage
+	 * @dataProvider quotaExceededProvider
+	 */
+	public function testCheckExceededQuota($quota, $headers)
+	{
+		$this->plugin->fileView = $this->buildFileViewMock($quota);
+
+		$this->server->httpRequest = new Sabre_HTTP_Request($headers);
+		$this->plugin->checkQuota('');
+	}
+
+	public function quotaOkayProvider() {
+		return array(
+			array(1024, array()),
+			array(1024, array('HTTP_X_EXPECTED_ENTITY_LENGTH' => '1024')),
+			array(1024, array('HTTP_CONTENT_LENGTH' => '512')),
+			array(1024, array('HTTP_OC_TOTAL_LENGTH' => '1024', 'HTTP_CONTENT_LENGTH' => '512')),
+			// OC\Files\FREE_SPACE_UNKNOWN = -2
+			array(-2, array()),
+			array(-2, array('HTTP_X_EXPECTED_ENTITY_LENGTH' => '1024')),
+			array(-2, array('HTTP_CONTENT_LENGTH' => '512')),
+			array(-2, array('HTTP_OC_TOTAL_LENGTH' => '1024', 'HTTP_CONTENT_LENGTH' => '512')),
+		);
+	}
+
+	public function quotaExceededProvider() {
+		return array(
+			array(1023, array('HTTP_X_EXPECTED_ENTITY_LENGTH' => '1024')),
+			array(511, array('HTTP_CONTENT_LENGTH' => '512')),
+			array(2047, array('HTTP_OC_TOTAL_LENGTH' => '2048', 'HTTP_CONTENT_LENGTH' => '1024')),
+		);
+	}
+
+	public function lengthProvider() {
+		return array(
+			array(null, array()),
+			array(1024, array('HTTP_X_EXPECTED_ENTITY_LENGTH' => '1024')),
+			array(512, array('HTTP_CONTENT_LENGTH' => '512')),
+			array(2048, array('HTTP_OC_TOTAL_LENGTH' => '2048', 'HTTP_CONTENT_LENGTH' => '1024')),
+			array(4096, array('HTTP_OC_TOTAL_LENGTH' => '2048', 'HTTP_X_EXPECTED_ENTITY_LENGTH' => '4096')),
+		);
+	}
+
+	private function buildFileViewMock($quota) {
+		// mock filesysten
+		$view = $this->getMock('\OC\Files\View', array('free_space'), array(), '', FALSE);
+		$view->expects($this->any())->method('free_space')->withAnyParameters()->will($this->returnValue($quota));
+
+		return $view;
+	}
+
+}
diff --git a/lib/connector/sabre/quotaplugin.php b/lib/connector/sabre/quotaplugin.php
index 34d4b676157404c010ec9564b744e000ada65a1e..ea2cb81d1f7532a79e86b1047a84ce2761f8297e 100644
--- a/lib/connector/sabre/quotaplugin.php
+++ b/lib/connector/sabre/quotaplugin.php
@@ -3,58 +3,95 @@
 /**
  * This plugin check user quota and deny creating files when they exceeds the quota.
  *
- * @copyright Copyright (C) 2012 entreCables S.L. All rights reserved.
  * @author Sergio Cambra
+ * @copyright Copyright (C) 2012 entreCables S.L. All rights reserved.
  * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
  */
 class OC_Connector_Sabre_QuotaPlugin extends Sabre_DAV_ServerPlugin {
 
 	/**
-		* Reference to main server object
-		*
-		* @var Sabre_DAV_Server
-		*/
+	 * Reference to main server object
+	 *
+	 * @var Sabre_DAV_Server
+	 */
 	private $server;
 
 	/**
-		* This initializes the plugin.
-		*
-		* This function is called by Sabre_DAV_Server, after
-		* addPlugin is called.
-		*
-		* This method should set up the requires event subscriptions.
-		*
-		* @param Sabre_DAV_Server $server
-		* @return void
-		*/
+	 * is kept public to allow overwrite for unit testing
+	 *
+	 * @var \OC\Files\View
+	 */
+	public $fileView;
+
+	/**
+	 * This initializes the plugin.
+	 *
+	 * This function is called by Sabre_DAV_Server, after
+	 * addPlugin is called.
+	 *
+	 * This method should set up the requires event subscriptions.
+	 *
+	 * @param Sabre_DAV_Server $server
+	 * @return void
+	 */
 	public function initialize(Sabre_DAV_Server $server) {
 
-			$this->server = $server;
-			$this->server->subscribeEvent('beforeWriteContent', array($this, 'checkQuota'), 10);
-			$this->server->subscribeEvent('beforeCreateFile', array($this, 'checkQuota'), 10);
+		$this->server = $server;
 
+		$server->subscribeEvent('beforeWriteContent', array($this, 'checkQuota'), 10);
+		$server->subscribeEvent('beforeCreateFile', array($this, 'checkQuota'), 10);
 	}
 
 	/**
-		* This method is called before any HTTP method and forces users to be authenticated
-		*
-		* @param string $method
-		* @throws Sabre_DAV_Exception
-		* @return bool
-		*/
+	 * This method is called before any HTTP method and validates there is enough free space to store the file
+	 *
+	 * @param string $method
+	 * @throws Sabre_DAV_Exception
+	 * @return bool
+	 */
 	public function checkQuota($uri, $data = null) {
-		$expected = $this->server->httpRequest->getHeader('X-Expected-Entity-Length');
-		$length = $expected ? $expected : $this->server->httpRequest->getHeader('Content-Length');
+		$length = $this->getLength();
 		if ($length) {
 			if (substr($uri, 0, 1)!=='/') {
 				$uri='/'.$uri;
 			}
 			list($parentUri, $newName) = Sabre_DAV_URLUtil::splitPath($uri);
-			$freeSpace = \OC\Files\Filesystem::free_space($parentUri);
+			$freeSpace = $this->getFreeSpace($parentUri);
 			if ($freeSpace !== \OC\Files\SPACE_UNKNOWN && $length > $freeSpace) {
 				throw new Sabre_DAV_Exception_InsufficientStorage();
 			}
 		}
 		return true;
 	}
+
+	public function getLength()
+	{
+		$req = $this->server->httpRequest;
+		$length = $req->getHeader('X-Expected-Entity-Length');
+		if (!$length) {
+			$length = $req->getHeader('Content-Length');
+		}
+
+		$ocLength = $req->getHeader('OC-Total-Length');
+		if ($length && $ocLength) {
+			return max($length, $ocLength);
+		}
+
+		return $length;
+	}
+
+	/**
+	 * @param $parentUri
+	 * @return mixed
+	 */
+	public function getFreeSpace($parentUri)
+	{
+		if (is_null($this->fileView)) {
+			// initialize fileView
+			$this->fileView = \OC\Files\Filesystem::getView();
+		}
+
+		$freeSpace = $this->fileView->free_space($parentUri);
+		return $freeSpace;
+	}
 }